import base64
from typing import Optional, Tuple, Union
from httpx import Request, Response
from cloudinary_analysis.httpclient import HttpClient

from .types import (
    BeforeRequestContext,
    BeforeRequestHook,
    AfterSuccessContext,
    AfterSuccessHook,
    AfterErrorContext,
    AfterErrorHook,
    SDKInitHook,
)

from .cloudinary_config import CloudConfig, AccountConfig


class CloudinaryCloudHook(SDKInitHook, BeforeRequestHook, AfterSuccessHook, AfterErrorHook):
    def __init__(self):
        # Initialize configurations
        self.cloud_config = CloudConfig()

    def sdk_init(self, base_url: str, client: HttpClient) -> Tuple[str, HttpClient]:
        return base_url, client

    def before_request(
        self, hook_ctx: BeforeRequestContext, request: Request
    ) -> Union[Request, Exception]:
        # Replace placeholder in the URL if cloud_name is configured
        if self.cloud_config.cloud_name and "/CLOUD_NAME/" in request.url.path:
            new_path = request.url.path.replace(
                "/CLOUD_NAME/", f"/{self.cloud_config.cloud_name}/"
            )
            request.url = request.url.copy_with(path=new_path)

        # Ensure security_source is valid and retrieve cloudinary_auth
        security = (
            hook_ctx.security_source.cloudinary_auth
            if hook_ctx.security_source
            and hasattr(hook_ctx.security_source, "cloudinary_auth")
            else None
        )

        # Get API credentials
        api_key = (
            security.api_key if security and hasattr(security, "api_key")
            else self.cloud_config.api_key
        )
        api_secret = (
            security.api_secret if security and hasattr(security, "api_secret")
            else self.cloud_config.api_secret
        )

        if not api_key or not api_secret:
            raise Exception("Cloudinary API key and secret are required")

        # Add Basic Authorization header
        data = f"{api_key}:{api_secret}".encode()
        request.headers["Authorization"] = f"Basic {base64.b64encode(data).decode()}"

        return request

    def after_success(
        self, hook_ctx: AfterSuccessContext, response: Response
    ) -> Union[Response, Exception]:
        return response

    def after_error(
        self,
        hook_ctx: AfterErrorContext,
        response: Optional[Response],
        error: Optional[Exception],
    ) -> Union[Tuple[Optional[Response], Optional[Exception]], Exception]:
        return response, error


class CloudinaryAccountHook(SDKInitHook, BeforeRequestHook, AfterSuccessHook, AfterErrorHook):
    def __init__(self):
        # Initialize configurations
        self.account_config = AccountConfig()

    def sdk_init(self, base_url: str, client: HttpClient) -> Tuple[str, HttpClient]:
        return base_url, client

    def before_request(
        self, hook_ctx: BeforeRequestContext, request: Request
    ) -> Union[Request, Exception]:
        # Replace placeholder in the URL if account_id is configured
        if self.account_config.account_id and "/ACCOUNT_ID/" in request.url.path:
            new_path = request.url.path.replace(
                "/ACCOUNT_ID/", f"/{self.account_config.account_id}/"
            )
            request.url = request.url.copy_with(path=new_path)

        # Ensure security_source is valid and retrieve cloudinary_account_auth
        security = (
            hook_ctx.security_source.cloudinary_account_auth
            if hook_ctx.security_source
            and hasattr(hook_ctx.security_source, "cloudinary_account_auth")
            else None
        )

        # Get provisioning credentials
        provisioning_api_key = (
            security.provisioning_api_key if security and hasattr(security, "provisioning_api_key")
            else self.account_config.provisioning_api_key
        )
        provisioning_api_secret = (
            security.provisioning_api_secret if security and hasattr(security, "provisioning_api_secret")
            else self.account_config.provisioning_api_secret
        )

        if not provisioning_api_key or not provisioning_api_secret:
            raise Exception("Provisioning API key and secret are required")

        # Add Basic Authorization header
        data = f"{provisioning_api_key}:{provisioning_api_secret}".encode()
        request.headers["Authorization"] = f"Basic {base64.b64encode(data).decode()}"

        return request

    def after_success(
        self, hook_ctx: AfterSuccessContext, response: Response
    ) -> Union[Response, Exception]:
        return response

    def after_error(
        self,
        hook_ctx: AfterErrorContext,
        response: Optional[Response],
        error: Optional[Exception],
    ) -> Union[Tuple[Optional[Response], Optional[Exception]], Exception]:
        return response, error
